import django_filters as filters
from django_filters import CharFilter, ModelChoiceFilter
from .models import Course, CourseRegistration, Invoice, Programme, Spreadsheet, Semester, Department
from django.db.models import Q

class CourseAcademicFilter(filters.FilterSet):
    course = CharFilter(field_name='course_code', lookup_expr='icontains', label="Course Code")
    class Meta:
        model = Course
        fields = []

class CourseUnderGradFilter(filters.FilterSet):
    course = CharFilter(field_name='course_code', lookup_expr='icontains', label="Course Code")
    course_title = CharFilter(field_name='course_title', lookup_expr='icontains', label="Course Title")
    class Meta:
        model = Course
        fields = []

class CourseRegisteredUnderGradFilter(filters.FilterSet):
    course = CharFilter(field_name='course__course_code', lookup_expr='icontains', label="Course Code")
    class Meta:
        model = CourseRegistration
        fields = []

class CourseRegisteredAcademicFilter(filters.FilterSet):
    course = CharFilter(field_name='course__course_code', lookup_expr='icontains', label="Course Code")
    undergrad = CharFilter(field_name='undergrad__username', lookup_expr='icontains', label="Matric Number")
    class Meta:
        model = CourseRegistration
        fields = []
    # def __init__(self, *args, **kwargs):
    #     super(RegisteredCourseUnderGradFilter, self).__init__(*args, **kwargs)
    #     self.filters['course__course_code'].label="Course Code"
    #     self.filters['undergrad__username'].label="Matric Number"

class InvoiceAdminFilter(filters.FilterSet):
    undergrad = CharFilter(field_name='undergrad__username', lookup_expr='icontains', label="Matric Number")
    class Meta:
        model = Invoice
        fields = []
    # def __init__(self, *args, **kwargs):
    #     super(RegisteredCourseUnderGradFilter, self).__init__(*args, **kwargs)
    #     self.filters['course__course_code'].label="Course Code"
    #     self.filters['undergrad__username'].label="Matric Number"

class ProgrammeFilter(filters.FilterSet):
    class Meta:
        model = Spreadsheet
        fields = ['semester', 'level', 'programme']

    def __init__(self, data=None, queryset=None, *, request=None, prefix=None):
        super().__init__(data=data, queryset=queryset, prefix=prefix)
        if request is not None and request.user.is_authenticated:
            self.filters['programme'].queryset = Programme.objects.filter(department__hod=request.user)

class ProgrammeFilterDean(filters.FilterSet):

    class Meta:
        model = Spreadsheet
        fields = ['semester', 'level', 'programme']

    def __init__(self, data=None, queryset=None, *, request=None, prefix=None):
        super().__init__(data=data, queryset=queryset, prefix=prefix)
        if request is not None and request.user.is_authenticated:
            self.filters['programme'].queryset = Programme.objects.filter(department__faculty__the_dean=request.user)

class ProgrammeFilterVC(filters.FilterSet):
    class Meta:
        model = Spreadsheet
        fields = ['semester', 'level', 'programme']

class CourseFilterHoD(filters.FilterSet):
    course_code = CharFilter(field_name='course_code', lookup_expr='icontains', label="Course Code")

    class Meta:
        model = Course
        fields = []

class CourseFilterDean(filters.FilterSet):
    course_code = CharFilter(field_name='course_code', lookup_expr='icontains', label="Course Code")

    class Meta:
        model = Course
        fields = []

class SemesterFilter(filters.FilterSet):
    class Meta:
        model = Semester
        fields = []
